@@ -1,14 +1,15 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
- * Copyright (C) 2005 Richard Hughes <hughsient@gmail.com>
  * Copyright (C) 2005 Jaap Haitsma <jaap@haitsma.org>
+ * Copyright (C) 2005 William Jon McCann <mccann@jhu.edu>
+ * Copyright (C) 2005-2006 Richard Hughes <richard@hughsie.com>
  *
  * Licensed under the GNU General Public License Version 2
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -17,11 +18,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
- * 02110-1301, USA.
- *
- * Authors: Richard Hughes <hughsient@gmail.com>
- *          Jaap Haitsma <jaap@haitsma.org>
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
 #include "config.h"
@@ -44,6 +41,7 @@
 #include "gpm-common.h"
 #include "gpm-hal.h"
 #include "gpm-prefs.h"
+#include "gpm-debug.h"
 
 static GConfEnumStringPair icon_policy_enum_map [] = {
        { GPM_ICON_POLICY_ALWAYS,       "always"   },
@@ -57,9 +55,10 @@ static GConfEnumStringPair icon_policy_enum_map [] = {
 #define ACTION_SUSPEND_TEXT		_("Suspend")
 #define ACTION_SHUTDOWN_TEXT		_("Shutdown")
 #define ACTION_HIBERNATE_TEXT		_("Hibernate")
+#define ACTION_BLANK_TEXT		_("Blank screen")
 #define ACTION_NOTHING_TEXT		_("Do nothing")
 
-#define	GPM_DBUS_SERVICE			"org.gnome.PowerManager"
+#define	GPM_DBUS_SERVICE		"org.gnome.PowerManager"
 #define	GPM_DBUS_PATH			"/org/gnome/PowerManager"
 #define	GPM_DBUS_INTERFACE	        "org.gnome.PowerManager"
 
@@ -80,7 +79,7 @@ gpm_dbus_method_bool (const char *method)
 	connection = dbus_g_bus_get (DBUS_BUS_SESSION, &error);
 	if (connection == NULL) {
 		if (error) {
-			g_warning ("Couldn't connect to PowerManager %s", error->message);
+			gpm_warning ("Couldn't connect to PowerManager %s", error->message);
 			g_error_free (error);
 		}
 		return FALSE;
@@ -97,7 +96,7 @@ gpm_dbus_method_bool (const char *method)
 				 G_TYPE_BOOLEAN, &value,
 				 G_TYPE_INVALID)) {
 		if (error) {
-			g_warning ("Couldn't connect to PowerManager %s", error->message);
+			gpm_warning ("Couldn't connect to PowerManager %s", error->message);
 			g_error_free (error);
 		}
 		value = FALSE;
@@ -140,16 +139,6 @@ gpm_has_lcd (void)
 	return value;
 }
 
-static gboolean
-gpm_has_button_sleep (void)
-{
-	gboolean value;
-
-	value = gpm_hal_num_devices_of_capability_with_value  ("button", "button.type", "sleep") > 0;
-
-	return value;
-}
-
 static gboolean
 gpm_has_button_lid (void)
 {
@@ -160,14 +149,6 @@ gpm_has_button_lid (void)
 	return value;
 }
 
-static void
-gpm_prefs_debug_log_ignore (const char *log_domain,
-			    GLogLevelFlags log_level,
-			    const char *message,
-			    gpointer user_data)
-{
-}
-
 static void
 gpm_prefs_help_cb (GtkWidget *widget,
 		   gpointer user_data)
@@ -177,7 +158,7 @@ gpm_prefs_help_cb (GtkWidget *widget,
 	gnome_help_display_with_doc_id (NULL, "gnome-power-manager",
 					"gnome-power-manager.xml", NULL, &error);
 	if (error != NULL) {
-		g_warning (error->message);
+		gpm_warning (error->message);
 		g_error_free (error);
 	}
 }
@@ -227,7 +208,7 @@ gpm_prefs_sleep_slider_changed_cb (GtkRange *range,
 {
 	int value;
 	GConfClient *client;
-	
+
 	value = (int)gtk_range_get_value (range);
 
 	if (value == NEVER_TIME_ON_SLIDER) {
@@ -251,17 +232,19 @@ gpm_prefs_setup_sleep_slider (GladeXML *dialog,
 	GtkWidget *widget;
 	gint value;
 	GConfClient *client;
+	gboolean is_writable;
 
 	widget = glade_xml_get_widget (dialog, widget_name);
-	g_signal_connect (G_OBJECT (widget), "format-value", 
+	g_signal_connect (G_OBJECT (widget), "format-value",
 			  G_CALLBACK (gpm_prefs_format_time_cb), NULL);
-	g_signal_connect (G_OBJECT (widget), "value-changed", 
-			  G_CALLBACK (gpm_prefs_sleep_slider_changed_cb), gpm_pref_key);
 
 	client = gconf_client_get_default ();
 	value = gconf_client_get_int (client, gpm_pref_key, NULL);
+	is_writable = gconf_client_key_is_writable (client, gpm_pref_key, NULL);
 	g_object_unref (client);
 
+	gtk_widget_set_sensitive (widget, is_writable);
+
 	if (value == 0) {
 		value = NEVER_TIME_ON_SLIDER;
 	} else {
@@ -271,6 +254,10 @@ gpm_prefs_setup_sleep_slider (GladeXML *dialog,
 
 	gtk_range_set_value (GTK_RANGE (widget), value);
 
+	g_signal_connect (G_OBJECT (widget), "value-changed",
+			  G_CALLBACK (gpm_prefs_sleep_slider_changed_cb),
+			  gpm_pref_key);
+
 	return widget;
 }
 
@@ -296,20 +283,25 @@ gpm_prefs_setup_brightness_slider (GladeXML *dialog,
 	GtkWidget *widget;
 	int value;
 	GConfClient *client;
+	gboolean is_writable;
 
 	widget = glade_xml_get_widget (dialog, widget_name);
 
-	g_signal_connect (G_OBJECT (widget), "format-value", 
+	g_signal_connect (G_OBJECT (widget), "format-value",
 			  G_CALLBACK (gpm_prefs_format_brightness_cb), NULL);
-	g_signal_connect (G_OBJECT (widget), "value-changed", 
-			  G_CALLBACK (gpm_prefs_brightness_slider_changed_cb), gpm_pref_key);
 
 	client = gconf_client_get_default ();
 	value = gconf_client_get_int (client, gpm_pref_key, NULL);
+	is_writable = gconf_client_key_is_writable (client, gpm_pref_key, NULL);
 	g_object_unref (client);
 
+	gtk_widget_set_sensitive (widget, is_writable);
+
 	gtk_range_set_value (GTK_RANGE (widget), value);
 
+	g_signal_connect (G_OBJECT (widget), "value-changed",
+			  G_CALLBACK (gpm_prefs_brightness_slider_changed_cb),
+			  gpm_pref_key);
 	return widget;
 }
 
@@ -326,15 +318,17 @@ gpm_prefs_action_combo_changed_cb (GtkWidget *widget,
 	if (strcmp (value, ACTION_SUSPEND_TEXT) == 0) {
 		action = ACTION_SUSPEND;
 	} else if (strcmp (value, ACTION_HIBERNATE_TEXT) == 0) {
-		action = ACTION_HIBERNATE;	
+		action = ACTION_HIBERNATE;
 	} else if (strcmp (value, ACTION_SHUTDOWN_TEXT) == 0) {
-		action = ACTION_SHUTDOWN;	
+		action = ACTION_SHUTDOWN;
+	} else if (strcmp (value, ACTION_BLANK_TEXT) == 0) {
+		action = ACTION_BLANK;
 	} else if (strcmp (value, ACTION_NOTHING_TEXT) == 0) {
-		action = ACTION_NOTHING;	
+		action = ACTION_NOTHING;
 	} else {
 		g_assert (FALSE);
 	}
-	
+
 	g_free (value);
 
 	client = gconf_client_get_default ();
@@ -342,43 +336,43 @@ gpm_prefs_action_combo_changed_cb (GtkWidget *widget,
 	g_object_unref (client);
 }
 
-static GtkWidget *
-gpm_prefs_setup_action_combo (GladeXML *dialog,
-			      char *widget_name,
+static void
+gpm_prefs_setup_action_combo (GtkWidget *widget,
 			      char *gpm_pref_key,
 			      const char **actions)
 {
-	GtkWidget *widget;
 	char *value;
 	int i = 0;
 	int n_added = 0;
 	gboolean can_suspend;
 	gboolean can_hibernate;
+	gboolean is_writable;
 	GConfClient *client;
 
 	can_suspend = gpm_can_suspend ();
 	can_hibernate = gpm_can_hibernate ();
 
-	widget = glade_xml_get_widget (dialog, widget_name);
-
 	client = gconf_client_get_default ();
 	value = gconf_client_get_string (client, gpm_pref_key, NULL);
+	is_writable = gconf_client_key_is_writable (client, gpm_pref_key, NULL);
 	g_object_unref (client);
 
+	gtk_widget_set_sensitive (widget, is_writable);
+
 	if (! value) {
-		g_warning ("gpm_prefs_setup_action_combo: invalid schema, please re-install");
-		return NULL;
+		gpm_warning ("invalid schema, please re-install");
+		value = g_strdup ("nothing");
 	}
 
-	while (actions[i] != NULL) { 	
+	while (actions[i] != NULL) {
 		if (strcmp (actions[i], ACTION_SHUTDOWN) == 0) {
 			gtk_combo_box_append_text (GTK_COMBO_BOX (widget),
 						   ACTION_SHUTDOWN_TEXT);
 			n_added++;
 		} else if ((strcmp (actions[i], ACTION_SUSPEND) == 0) && !can_suspend) {
-			g_debug ("Cannot add option, as cannot suspend.");
+			gpm_debug ("Cannot add option, as cannot suspend.");
 		} else if ((strcmp (actions[i], ACTION_HIBERNATE) == 0) && !can_hibernate) {
-			g_debug ("Cannot add option, as cannot hibernate.");
+			gpm_debug ("Cannot add option, as cannot hibernate.");
 		} else if ((strcmp (actions[i], ACTION_SUSPEND) == 0) && can_suspend) {
 			gtk_combo_box_append_text (GTK_COMBO_BOX (widget),
 						   ACTION_SUSPEND_TEXT);
@@ -387,6 +381,10 @@ gpm_prefs_setup_action_combo (GladeXML *dialog,
 			gtk_combo_box_append_text (GTK_COMBO_BOX (widget),
 						   ACTION_HIBERNATE_TEXT);
 			n_added++;
+		} else if (strcmp (actions[i], ACTION_BLANK) == 0) {
+			gtk_combo_box_append_text (GTK_COMBO_BOX (widget),
+						   ACTION_BLANK_TEXT);
+			n_added++;
 		} else if (strcmp (actions[i], ACTION_NOTHING) == 0) {
 			gtk_combo_box_append_text (GTK_COMBO_BOX (widget),
 						   ACTION_NOTHING_TEXT);
@@ -400,49 +398,68 @@ gpm_prefs_setup_action_combo (GladeXML *dialog,
 		i++;
 	}
 	g_signal_connect (G_OBJECT (widget), "changed",
-			  G_CALLBACK (gpm_prefs_action_combo_changed_cb), gpm_pref_key);
+			  G_CALLBACK (gpm_prefs_action_combo_changed_cb),
+			  gpm_pref_key);
 
 	g_free (value);
-
-	return widget;
 }
 
+static void
+gpm_prefs_checkbox_lock_cb (GtkWidget  *widget,
+			    const char *gpm_pref_key)
+{
+	GConfClient *client;
+	gboolean checked;
+
+	client = gconf_client_get_default ();
+
+	checked = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget));
+
+	gpm_debug ("Changing %s to %i", gpm_pref_key, checked);
+
+	gconf_client_set_bool (client,
+			       gpm_pref_key,
+			       checked, NULL);
+
+	g_object_unref (client);
+}
 
 static void
-setup_page_one (GladeXML *xml)
+gpm_prefs_setup_checkbox (GtkWidget *widget,
+			  char	    *gpm_pref_key)
 {
-	GtkWidget   *widget;
+
 	GConfClient *client;
-	GtkWidget   *label_ac_brightness;
-	GtkWidget   *slider_ac_brightness;
+	gboolean checked;
+
+	gpm_debug ("Setting up %s", gpm_pref_key);
+	client = gconf_client_get_default ();
+
+	checked = gconf_client_get_bool (client, gpm_pref_key, NULL);
+
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), checked);
+
+	g_signal_connect (widget, "clicked",
+			  G_CALLBACK (gpm_prefs_checkbox_lock_cb),
+			  (gpointer) gpm_pref_key);
+
+	g_object_unref (client);
+}
+
+static void
+setup_battery_sliders (GladeXML *xml, gboolean has_batteries)
+{
 	GtkWidget   *label_batteries_display;
 	GtkWidget   *slider_batteries_display;
 	GtkWidget   *label_batteries_brightness;
 	GtkWidget   *slider_batteries_brightness;
-	gboolean     has_batteries;
 	gboolean     can_set_brightness;
 
-	client = gconf_client_get_default ();
+	if (! has_batteries) {
+		/* no point */
+		return;
+	}
 
-	/* AC icon */
-	widget = glade_xml_get_widget (xml, "image_side_acadapter");
-	gtk_image_set_from_icon_name (GTK_IMAGE (widget), "gnome-fs-socket", GTK_ICON_SIZE_DIALOG);
-	
-	/* Sleep time on AC */
-	gpm_prefs_setup_sleep_slider (xml, "hscale_ac_computer", GPM_PREF_AC_SLEEP_COMPUTER);
-
-	/* Sleep time for display on AC */	
-	gpm_prefs_setup_sleep_slider (xml, "hscale_ac_display", GPM_PREF_AC_SLEEP_DISPLAY);
-
-	/* Display brightness when on AC */
-	label_ac_brightness = glade_xml_get_widget (xml, "label_ac_brightness");
-	slider_ac_brightness = gpm_prefs_setup_brightness_slider (xml, "hscale_ac_brightness",
-								  GPM_PREF_AC_BRIGHTNESS);
-
-	/* Battery icon */
-	widget = glade_xml_get_widget (xml, "image_side_battery");
-	gtk_image_set_from_icon_name (GTK_IMAGE (widget), "gnome-dev-battery", GTK_ICON_SIZE_DIALOG);
-	
 	/* Sleep time on batteries */
 	gpm_prefs_setup_sleep_slider (xml, "hscale_batteries_computer", GPM_PREF_BATTERY_SLEEP_COMPUTER);
 
@@ -457,129 +474,156 @@ setup_page_one (GladeXML *xml)
 	label_batteries_brightness = glade_xml_get_widget (xml, "label_batteries_brightness");
 	slider_batteries_brightness = gpm_prefs_setup_brightness_slider (xml, "hscale_batteries_brightness",
 									 GPM_PREF_BATTERY_BRIGHTNESS);
+	can_set_brightness = gpm_has_lcd ();
+	if (! can_set_brightness) {
+		gtk_widget_hide_all (label_batteries_brightness);
+		gtk_widget_hide_all (slider_batteries_brightness);
+	}
 
-	has_batteries = gpm_has_batteries ();
+}
+
+static void
+setup_ac_sliders (GladeXML *xml, gboolean has_batteries)
+{
+	GtkWidget   *widget;
+	GtkWidget   *label_ac_brightness;
+	GtkWidget   *slider_ac_brightness;
+	gboolean     can_set_brightness;
+
+	/* Sleep time on AC */
+	gpm_prefs_setup_sleep_slider (xml, "hscale_ac_computer", GPM_PREF_AC_SLEEP_COMPUTER);
+
+	/* Sleep time for display on AC */
+	gpm_prefs_setup_sleep_slider (xml, "hscale_ac_display", GPM_PREF_AC_SLEEP_DISPLAY);
+
+	/* Display brightness when on AC */
+	label_ac_brightness = glade_xml_get_widget (xml, "label_ac_brightness");
+	slider_ac_brightness = gpm_prefs_setup_brightness_slider (xml, "hscale_ac_brightness",
+								  GPM_PREF_AC_BRIGHTNESS);
+
+	can_set_brightness = gpm_has_lcd ();
+	if (! can_set_brightness) {
+		gtk_widget_hide_all (label_ac_brightness);
+		gtk_widget_hide_all (slider_ac_brightness);
+	}
+
+	/* when we have no batteries, we just want the title to be "Configuration" */
 	if (! has_batteries) {
 		char *str;
 		widget = glade_xml_get_widget (xml, "label_frame_ac");
 		str = g_strdup_printf ("<b>%s</b>", _("Configuration"));
 		gtk_label_set_markup (GTK_LABEL (widget), str);
 		g_free (str);
-
-		/* Sleep tab */
-		widget = glade_xml_get_widget (xml, "frame_batteries");
-		gtk_widget_hide_all (widget);
 	}
-
-	can_set_brightness = gpm_has_lcd ();
-	if (! can_set_brightness) {
-		gtk_widget_hide_all (label_ac_brightness);
-		gtk_widget_hide_all (slider_ac_brightness);
-		gtk_widget_hide_all (label_batteries_brightness);
-		gtk_widget_hide_all (slider_batteries_brightness);
-	}
-
-	g_object_unref (client);
 }
 
 static void
-setup_page_two (GladeXML *xml)
+setup_sleep_type (GladeXML *xml)
 {
-	GtkWidget    *widget;
-	GConfClient  *client;
-	GtkSizeGroup *size_group;
-	const char   *sleep_type_actions[] = {ACTION_SUSPEND, ACTION_HIBERNATE, NULL};
-	const char   *button_suspend_actions[] = {ACTION_NOTHING, ACTION_SUSPEND, ACTION_HIBERNATE, NULL};
-	const char   *button_lid_actions[] = {ACTION_NOTHING, ACTION_SUSPEND, ACTION_HIBERNATE, NULL};
-	const char   *battery_critical_actions[] = {ACTION_NOTHING, ACTION_HIBERNATE, ACTION_SHUTDOWN, NULL};
 	GtkWidget    *label_sleep_type;
 	GtkWidget    *combo_sleep_type;
+	GtkWidget    *checkbutton_dim_idle;
+	gboolean      can_set_brightness;
+	const char   *sleep_type_actions[] = {ACTION_NOTHING, ACTION_SUSPEND, ACTION_HIBERNATE, NULL};
+
+	/* Sleep Type Combo Box */
+	label_sleep_type = glade_xml_get_widget (xml, "label_sleep_type");
+	combo_sleep_type = glade_xml_get_widget (xml, "combobox_sleep_type");
+
+	gpm_prefs_setup_action_combo (combo_sleep_type,
+				      GPM_PREF_SLEEP_TYPE,
+				      sleep_type_actions);
+
+	/* set up the "do we dim screen on idle checkbox */
+	checkbutton_dim_idle = glade_xml_get_widget (xml, "checkbutton_dim_idle");
+	can_set_brightness = gpm_has_lcd ();
+	if (can_set_brightness) {
+		gpm_prefs_setup_checkbox (checkbutton_dim_idle,
+					  GPM_PREF_IDLE_DIM_SCREEN);
+	} else {
+		gtk_widget_hide_all (checkbutton_dim_idle);
+	}
+}
+
+static void
+setup_ac_actions (GladeXML *xml)
+{
+	GtkWidget    *label_button_lid;
+	GtkWidget    *combo_button_lid;
+	GtkWidget    *vbox_ac_actions;
+	gboolean      has_lid_button;
+	const char   *button_lid_actions[] = {ACTION_BLANK, ACTION_SUSPEND, ACTION_HIBERNATE, NULL};
+
+	label_button_lid = glade_xml_get_widget (xml, "label_ac_button_lid");
+	combo_button_lid = glade_xml_get_widget (xml, "combobox_ac_lid_close");
+	vbox_ac_actions = glade_xml_get_widget (xml, "vbox_ac_actions");
+
+	has_lid_button = gpm_has_button_lid ();
+
+	if (has_lid_button) {
+		gpm_prefs_setup_action_combo (combo_button_lid,
+					      GPM_PREF_AC_BUTTON_LID,
+					      button_lid_actions);
+	} else {
+		gtk_widget_hide_all (label_button_lid);
+		gtk_widget_hide_all (combo_button_lid);
+		/* there is nothing else in this action box apart from
+		   lid event, so hide the whole box */
+		gtk_widget_hide_all (vbox_ac_actions);
+	}
+}
+
+static void
+setup_battery_actions (GladeXML *xml, gboolean has_batteries)
+{
 	GtkWidget    *label_button_lid;
 	GtkWidget    *combo_button_lid;
 	GtkWidget    *label_battery_critical;
 	GtkWidget    *combo_battery_critical;
-	GtkWidget    *label_button_suspend;
-	GtkWidget    *combo_button_suspend;
-	gboolean      has_batteries;
-	gboolean      has_suspend_button;
+	const char   *button_lid_actions[] = {ACTION_BLANK, ACTION_SUSPEND, ACTION_HIBERNATE, NULL};
+	const char   *battery_critical_actions[] = {ACTION_NOTHING, ACTION_SUSPEND, ACTION_HIBERNATE, ACTION_SHUTDOWN, NULL};
 	gboolean      has_lid_button;
 
-	client = gconf_client_get_default ();
-
-	/* Sleep Type Combo Box */
-
-	label_sleep_type = glade_xml_get_widget (xml, "label_sleep_type");
-	combo_sleep_type = gpm_prefs_setup_action_combo (xml, "combobox_sleep_type",
-							 GPM_PREF_SLEEP_TYPE, sleep_type_actions);
-
-	/* Button Suspend Combo Box */
-
-	label_button_suspend = glade_xml_get_widget (xml, "label_button_suspend");
-	combo_button_suspend = gpm_prefs_setup_action_combo (xml, "combobox_button_suspend",
-				      			     GPM_PREF_BUTTON_SUSPEND, button_suspend_actions);
-	/* Button Lid Combo Box */
-
-	label_button_lid = glade_xml_get_widget (xml, "label_button_lid");
-	combo_button_lid = gpm_prefs_setup_action_combo (xml, "combobox_button_lid",
-				      			 GPM_PREF_BUTTON_LID, button_lid_actions);
-
-	/* Battery critical Combo Box */
-
-	label_battery_critical = glade_xml_get_widget (xml, "label_battery_critical_action");
-	combo_battery_critical = gpm_prefs_setup_action_combo (xml, "combobox_battery_critical",
-							       GPM_PREF_BATTERY_CRITICAL, battery_critical_actions);
-
-	/* Make sure that all comboboxes get the same size by adding their
-	 * labels to a GtkSizeGroup
-	 */
-	size_group = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
-	gtk_size_group_add_widget (size_group, label_sleep_type);
-	gtk_size_group_add_widget (size_group, label_button_suspend);
-	gtk_size_group_add_widget (size_group, label_button_lid);
-	gtk_size_group_add_widget (size_group, label_battery_critical);
-	g_object_unref (G_OBJECT (size_group));
-
-
-	has_suspend_button = gpm_has_button_sleep ();
-	if (! has_suspend_button) {
-		gtk_widget_hide_all (label_button_suspend);
-		gtk_widget_hide_all (combo_button_suspend);
+	if (! has_batteries) {
+		/* no point */
+		return;
 	}
 
+	/* Button Lid Combo Box */
+	label_button_lid = glade_xml_get_widget (xml, "label_battery_button_lid");
+	combo_button_lid = glade_xml_get_widget (xml, "combobox_battery_lid_close");
+
 	has_lid_button = gpm_has_button_lid ();
-	if (! has_lid_button) {
+
+	if (has_lid_button) {
+		gpm_prefs_setup_action_combo (combo_button_lid,
+					      GPM_PREF_BATTERY_BUTTON_LID,
+					      button_lid_actions);
+	} else {
 		gtk_widget_hide_all (label_button_lid);
 		gtk_widget_hide_all (combo_button_lid);
 	}
+	/* FIXME: also need lid lock */
 
-	has_batteries = gpm_has_batteries ();
+	label_battery_critical = glade_xml_get_widget (xml, "label_battery_critical_action");
+	combo_battery_critical = glade_xml_get_widget (xml, "combobox_battery_critical");
 
-	/* if no options then disable frame as it will be empty */
-	if (! has_batteries && ! has_suspend_button && ! has_lid_button) {
-		widget = glade_xml_get_widget (xml, "frame_actions");
-		gtk_widget_hide_all (widget);
-	}
-
-	if (! has_batteries) {
-		/* Hide battery options in options tab */
-		gtk_widget_hide_all (label_battery_critical);
-		gtk_widget_hide_all (combo_battery_critical);
-	}
-
-	g_object_unref (client);
+	gpm_prefs_setup_action_combo (combo_battery_critical,
+				      GPM_PREF_BATTERY_CRITICAL,
+				      battery_critical_actions);
 }
 
 static void
-setup_page_three (GladeXML *xml)
+setup_icon_policy (GladeXML *xml, gboolean has_batteries)
 {
 	GConfClient *client;
-	gboolean     has_batteries;
 	char        *icon_policy_str;
 	int          icon_policy;
 	GtkWidget   *radiobutton_icon_always;
 	GtkWidget   *radiobutton_icon_charge;
 	GtkWidget   *radiobutton_icon_critical;
 	GtkWidget   *radiobutton_icon_never;
+	gboolean     is_writable;
 
 	client = gconf_client_get_default ();
 
@@ -593,6 +637,23 @@ setup_page_three (GladeXML *xml)
 	radiobutton_icon_critical = glade_xml_get_widget (xml, "radiobutton_icon_critical");
 	radiobutton_icon_never = glade_xml_get_widget (xml, "radiobutton_icon_never");
 
+	is_writable = gconf_client_key_is_writable (client, GPM_PREF_ICON_POLICY, NULL);
+	gtk_widget_set_sensitive (radiobutton_icon_always, is_writable);
+	gtk_widget_set_sensitive (radiobutton_icon_charge, is_writable);
+	gtk_widget_set_sensitive (radiobutton_icon_critical, is_writable);
+	gtk_widget_set_sensitive (radiobutton_icon_never, is_writable);
+
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radiobutton_icon_always),
+				      icon_policy == GPM_ICON_POLICY_ALWAYS);
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radiobutton_icon_charge),
+				      icon_policy == GPM_ICON_POLICY_CHARGE);
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radiobutton_icon_critical),
+				      icon_policy == GPM_ICON_POLICY_CRITICAL);
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radiobutton_icon_never),
+				      icon_policy == GPM_ICON_POLICY_NEVER);
+
+	/* only connect the callbacks after we set the value, else the gconf
+	   keys gets written to (for a split second), and the icon flickers. */
 	g_signal_connect (radiobutton_icon_always, "clicked",
 			  G_CALLBACK (gpm_prefs_icon_radio_cb),
 			  (gpointer)GPM_ICON_POLICY_ALWAYS);
@@ -606,18 +667,8 @@ setup_page_three (GladeXML *xml)
 			  G_CALLBACK (gpm_prefs_icon_radio_cb),
 			  (gpointer)GPM_ICON_POLICY_NEVER);
 
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radiobutton_icon_always),
-				      icon_policy == GPM_ICON_POLICY_ALWAYS);
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radiobutton_icon_charge),
-				      icon_policy == GPM_ICON_POLICY_CHARGE);
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radiobutton_icon_critical),
-				      icon_policy == GPM_ICON_POLICY_CRITICAL);
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radiobutton_icon_never),
-				      icon_policy == GPM_ICON_POLICY_NEVER);
-
-	has_batteries = gpm_has_batteries ();
 	if (! has_batteries) {
-		/* Hide battery radio options in advanced tab */
+		/* Hide battery radio options if we have no batteries */
 		gtk_widget_hide_all (radiobutton_icon_charge);
 		gtk_widget_hide_all (radiobutton_icon_critical);
 	}
@@ -628,9 +679,10 @@ setup_page_three (GladeXML *xml)
 static GtkWidget *
 gpm_prefs_create (void)
 {
-	GtkWidget *main_window;
-	GtkWidget *widget;
-	GladeXML *glade_xml;
+	GtkWidget    *main_window;
+	GtkWidget    *widget;
+	GladeXML     *glade_xml;
+	gboolean      has_batteries;
 
 	glade_xml = glade_xml_new (GPM_DATA "/gpm-prefs.glade", NULL, NULL);
 
@@ -653,10 +705,20 @@ gpm_prefs_create (void)
 	g_signal_connect (widget, "clicked",
 			  G_CALLBACK (gpm_prefs_help_cb), NULL);
 
-	setup_page_one (glade_xml);
-	setup_page_two (glade_xml);
-	setup_page_three (glade_xml);
+	has_batteries = gpm_has_batteries ();
 
+	setup_icon_policy (glade_xml, has_batteries);
+	setup_ac_actions (glade_xml);
+	setup_ac_sliders (glade_xml, has_batteries);
+	setup_battery_actions (glade_xml, has_batteries);
+	setup_battery_sliders (glade_xml, has_batteries);
+	setup_sleep_type (glade_xml);
+
+	/* if no options then disable frame as it will be empty */
+	if (! has_batteries) {
+		widget = glade_xml_get_widget (glade_xml, "gpm_notebook");
+		gtk_notebook_remove_page (GTK_NOTEBOOK(widget), 1);
+	}
 	return main_window;
 }
 
@@ -689,14 +751,14 @@ main (int argc, char **argv)
 	bind_textdomain_codeset (PACKAGE, "UTF-8");
 	textdomain (PACKAGE);
 
-	if (! verbose) {
-		g_log_set_handler (G_LOG_DOMAIN, G_LOG_LEVEL_DEBUG, gpm_prefs_debug_log_ignore, NULL);
-	}
+	gpm_debug_init (verbose);
 
 	dialog = gpm_prefs_create ();
 	gtk_widget_show (dialog);
 
 	gtk_main ();
 
+	gpm_debug_shutdown ();
+
 	return 0;
 }
