@@ -36,6 +36,8 @@ G_BEGIN_DECLS
 
 #define GPM_PREF_SLEEP_TYPE		GPM_PREF_DIR "/action_sleep_type"
 #define GPM_PREF_BUTTON_SUSPEND		GPM_PREF_DIR "/action_button_suspend"
+#define GPM_PREF_BUTTON_HIBERNATE	GPM_PREF_DIR "/action_button_hibernate"
+#define GPM_PREF_BUTTON_POWER		GPM_PREF_DIR "/action_button_power"
 #define GPM_PREF_AC_BUTTON_LID		GPM_PREF_DIR "/action_ac_button_lid"
 #define GPM_PREF_BATTERY_BUTTON_LID	GPM_PREF_DIR "/action_battery_button_lid"
 #define GPM_PREF_BATTERY_CRITICAL	GPM_PREF_DIR "/action_battery_critical"
@@ -72,6 +74,7 @@ typedef enum {
 #define ACTION_HIBERNATE		"hibernate"
 #define ACTION_BLANK			"blank"
 #define ACTION_NOTHING			"nothing"
+#define ACTION_INTERACTIVE		"interactive"
 
 G_END_DECLS
 
